#!/bin/bash
OUTTER_NAME="NNRT"
PACKAGE_SHORT_NAME="nnrt"
PACKAGE_VERSION_FORM="x86_64-linux"
INSTALL_DIRECTORY="nnrt"
PACKAGE_VERSION="6.2.RC2"
PACKAGE_NAME="Ascend-cann-nnrt_6.2.RC2_linux-x86_64.run"
PKG_VERSION="base"
PACKAGE_DESCRIPTION="DaVinci AI Software"

if [ "$UID" = "0" ]; then
    DEFAULT_INSTALL_PATH="/usr/local/Ascend"
else
    DEFAULT_INSTALL_PATH="${HOME}/Ascend"
fi

LOG_PATH="${1}"
LOG_PROGRESS_NAME="progress_${OUTTER_NAME}.txt"
LOG_COMMON_NAME="ascend_${OUTTER_NAME}.log"
LOG_PROGRESS_FILE="${LOG_PATH}/${LOG_PROGRESS_NAME}"
LOG_COMMON_FILE="${LOG_PATH}/${LOG_COMMON_NAME}"

uninstall_flag=n
showinfo_flag=n
type_param=""

# 路径
CUR_PATH="$(
    cd "$(dirname "$0")"
    pwd
)"
CUR_PATH="$(readlink -f $CUR_PATH)"
PACKAGE_FILE=$CUR_PATH/${PACKAGE_NAME}

function print() {
    # 将关键信息打印到屏幕上
    if [ ! -f $LOG_COMMON_FILE ]; then
        echo "[${OUTTER_NAME}] [$(date +%Y%m%d-%H:%M:%S)] [$1] $2"
    else
        echo "[${OUTTER_NAME}] [$(date +%Y%m%d-%H:%M:%S)] [$1] $2" | tee -a $LOG_COMMON_FILE
    fi
}

### 脚本入参的相关处理函数
function parse_script_args() {
    if [[ "${OUTTER_NAME}" =~ "kernels" ]]; then
        if [[ $# -gt 2 ]] || [[ $# -lt 1 ]]; then
            print "ERROR" "parameter error !"
            exit 128
        fi
        if [[ $# -eq 2 ]] && [ "$2" != "nnae" -a "$2" != "toolkit" ]; then
            print "ERROR" "the parameter [$2] is not nnae or toolkit. Please check it ."
            exit 128
        elif [[ $# -eq 2 ]]; then
            type_param="--type=${2}"
        fi
    fi
    if [ $# -ne 1 ] && [[ ! "${OUTTER_NAME}" =~ "kernels" ]]; then
        print "ERROR" "parameter error !"
        exit 128
    fi
    case "$1" in
        show_info)
            showinfo_flag=y
            ;;
        uninstall)
            uninstall_flag=y
            ;;
    esac
}

function log_init() {
    if [ ! -d ${LOG_PATH} ]; then
        mkdir -p ${LOG_PATH}
    fi
}

function show_info() {
    check_install ${DEFAULT_INSTALL_PATH}
    if [ $? -eq 1 ]; then
        echo "Name": "${OUTTER_NAME}"
        echo "Version": "${PACKAGE_VERSION}"
        echo "Description": "${PACKAGE_DESCRIPTION}"
        exit 0
    else
        exit 2
    fi
}

function check_install() {
    local install_path=$1
    # 针对二进制算子的检测
    if [[ "${OUTTER_NAME}" =~ "kernels" ]]; then
        local kernel_version_num=$(ls ${install_path}/*/*/opp/built-in/op_impl/ai_core/tbe/kernel/version.info 2>/dev/null | wc -l)
        if [ ${kernel_version_num} -gt 0 ]; then
            return 1
        else
            return 0
        fi
    fi
    local installed_info_num=$(ls ${install_path}/${INSTALL_DIRECTORY}/*/${PACKAGE_VERSION_FORM}/ascend_${PACKAGE_SHORT_NAME}_install.info 2>/dev/null | wc -l)
    # 路径中只存在同架构的一个版本，升级
    if [ ${installed_info_num} -gt 0 ]; then
        return 1
    # 路径中未安装同架构任何版本，安装
    elif [ ${installed_info_num} -eq 0 ]; then
        return 0
    fi
}

function register_information_to_ibma() {
    local IBMA_CONFIG_PATH="/opt/huawei/ibma/config"
    local IBMA_EXRESOURCE_PATH="${IBMA_CONFIG_PATH}/exresource"
    local RESOURCE_FILE="${IBMA_EXRESOURCE_PATH}/ExternalResource.ini"

    if [ ! -d "${IBMA_CONFIG_PATH}" ]; then
        print "INFO" "iBMA is not installed."
        return 0
    fi
    if [ ! -d "${IBMA_EXRESOURCE_PATH}" ]; then
        mkdir -pm 750 "${IBMA_EXRESOURCE_PATH}"
    fi
    cp -f "${CUR_PATH}/${OUTTER_NAME}.sh" "${IBMA_EXRESOURCE_PATH}"
    chmod 540 "${IBMA_EXRESOURCE_PATH}/${OUTTER_NAME}.sh"
    if [ -f "${RESOURCE_FILE}" ]; then
        grep -w "${OUTTER_NAME}" "${RESOURCE_FILE}" >& /dev/null
        if [ $? -ne 0 ]; then
            sed -i "s/]}/,\"${OUTTER_NAME}\"]}/g" "${RESOURCE_FILE}"
        fi
    else
        echo  "{\"Software\": [\"${OUTTER_NAME}\"]}" > "${RESOURCE_FILE}"
    fi
    print "INFO" "The software information ${OUTTER_NAME} is successfully registered with the iBMA."
    return 0
}

function install_process() {
    log_init
    local message_info=$1
    chmod +x ${PACKAGE_FILE}
    bash ${PACKAGE_FILE} $2 "--nox11" "${type_param}" >>${LOG_COMMON_FILE}
    # 进度条的显示需要确认
    if [ $? -eq 0 ]; then
        echo "{\"name\":\"${OUTTER_NAME}\", \"progress\":100, \"message\":\"${OUTTER_NAME} is ${message_info} complete !\"}" | tee -a "${LOG_PROGRESS_FILE}"
        print "INFO" "$1 success !"
        exit 0
    else
        echo "{\"name\":\"${OUTTER_NAME}\", \"progress\":100, \"message\":\"${OUTTER_NAME} is ${message_info} failed !\"}" | tee -a "${LOG_PROGRESS_FILE}"
        print "ERROR" "$1 failed !"
        exit 127
    fi
}

function uninstall_process() {
    chmod +x ${PACKAGE_FILE}
    bash ${PACKAGE_FILE} "--uninstall" "--nox11" "${type_param}"
    if [ $? -eq 0 ]; then
        print "INFO" "uninstall success !"
        exit 0
    else
        print "ERROR" "uninstall failed !"
        exit 127
    fi
}

function process() {
    if [ ${showinfo_flag} == "y" ]; then
        show_info
    elif [ ${uninstall_flag} == "y" ]; then
        uninstall_process
    else
        check_install ${DEFAULT_INSTALL_PATH}
        if [ $? -eq 0 ] || [[ "${OUTTER_NAME}" =~ "kernels" ]]; then
            install_process "install" "--install"
        elif [ "${PKG_VERSION}" == "base" ]; then
            install_process "upgrade" "--upgrade"
        elif [ "${PKG_VERSION}" == "patch" ]; then
            install_process "upgrade" "--install"
        fi
    fi
}

# 程序开始
function main() {
    parse_script_args $*
    process $*
}

main $*
